@@ -11,8 +11,9 @@ Simple interface, simple deploy.
 
 """
 
-# import mf
+import mf
 from dns import resolver as dns
+from webagt import ConnectionError, get
 
 from . import host
 from .framework import *  # noqa
