@@ -41,18 +41,6 @@ def update_config(**items):
     return get_config()
 
 
-@main.register()
-class MF:
-    """Get a resource and parse it for Microformats."""
-
-    def setup(self, add_arg):
-        add_arg("uri", help="address of the resource")
-
-    def run(self, stdin, log):
-        pprint(webagt.get(self.uri).mf2json.data)
-        return 0
-
-
 @main.register()
 class Apps:
     """Show installed web apps."""
@@ -136,16 +124,8 @@ class Config:
 
 
 @main.register()
-class Scaffold:
-    """"""
-
-    def run(self, stdin, log):
-        return 0
-
-
-@main.register()
-class Publish:
-    """Publish a web app."""
+class Bootstrap:
+    """Bootstrap a cloud machine."""
 
     def run(self, stdin, log):
         logging.basicConfig(
@@ -188,13 +168,19 @@ class Publish:
                 machine._apt("update")
                 machine._apt("install -yq ufw")
                 machine.open_ports(22)
-                console.print(f"Initializing nginx ([magenta]1 min[/magenta])")
+                console.print(f"Installing tor..")
+                protected_path = machine.setup_tor()
+                console.print(f"Installing nginx.. ([magenta]1 min[/magenta])")
                 protected_path = machine.setup_nginx()
                 webbrowser.open(f"http://{machine.address}/{protected_path}")
-                console.print("Setting up base system ([magenta]3 min[/magenta])")
+                console.print("Setting up system.. ([magenta]3 min[/magenta])")
                 machine.setup_machine()
-                console.print(f"Installing python ([magenta]5 min[/magenta])")
+                console.print(f"Installing python.. ([magenta]5 min[/magenta])")
                 machine.setup_python()
+                console.print(
+                    f"Installing admin interface.. ([magenta]5 min[/magenta])"
+                )
+                machine.setup_admin()
                 machines.append(machine.address)
                 update_config(machines=machines)
         machine = web.host.Machine(machines[0], "root", "gaea_key")
