@@ -48,7 +48,7 @@ import toml
 import unidecode
 import watchdog.events
 import watchdog.observers
-import webcli
+import webagt
 from rich.console import Console
 
 from newmath import nb60_re, nbdecode, nbencode, nbrandom
@@ -679,7 +679,7 @@ class Response:
     @property
     def dom(self):
         if self.parsed is None:
-            self.parsed = webcli.parse(self.body)
+            self.parsed = webagt.parse(self.body)
         return self.parsed
 
 
@@ -1111,7 +1111,7 @@ class Application:
         ):
             handling_app = tx.request.controller.__web__[2]
             if getattr(handling_app, "parent_app", None):
-                dom = webcli.parse(tx.response.body)
+                dom = webagt.parse(tx.response.body)
 
                 def linkify(link):
                     if link.startswith(
