@@ -8,7 +8,9 @@ import getpass
 import io
 import logging
 import os
+import pathlib
 import subprocess
+import tempfile
 import textwrap
 import time
 from contextlib import contextmanager
@@ -110,6 +112,10 @@ class MachineBase:
                     for line in self.lines:
                         yield line
 
+                @property
+                def stdout(self):
+                    return "\n".join(self.lines)
+
             process = Process()
             key_args = []
             if self.key:
@@ -161,6 +167,25 @@ class MachineBase:
             tries -= 1
         raise ConnectionError("SSH connection could not be made")
 
+    def get(self, from_path, to_path=None):
+        """"""
+
+        def cp():
+            self.cp(f"{self.user}@{self.address}:{from_path}", to_path)
+
+        if to_path:
+            cp()
+        else:
+            with tempfile.TemporaryDirectory() as tmpdirname:
+                to_path = pathlib.Path(tmpdirname) / "alpha"
+                cp()
+                with to_path.open() as fp:
+                    return fp.read()
+
+    def send(self, from_path, to_path):
+        """"""
+        return self.cp(from_path, f"{self.user}@{self.address}:{to_path}")
+
     def cp(self, from_path, to_path):
         """Return a function for sending/retrieving a file over SCP."""
         with subprocess.Popen(
@@ -281,7 +306,6 @@ class Machine(MachineBase):
         "libsm-dev",
         # TODO "python3-opencv",  # computer vision  # NOTE lots of packages...
         # XXX "libevent-dev",  # Tor
-        "tor",
         "pandoc",  # markup translation
         "graphviz",  # graphing
         "brotli",  # wasm build for DT
@@ -299,8 +323,6 @@ class Machine(MachineBase):
     )
     versions = {
         "python": "3.10.9",
-        # "nginx": "1.23.3",
-        # "tor": "0.4.6.10",
         # "firefox": "97.0",
         # "geckodriver": "0.27.0",
     }
@@ -316,7 +338,6 @@ class Machine(MachineBase):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.tor_dir = self.system_dir / "tor"
         self.nginx_dir = self.system_dir / "nginx"
         self.python_dir = self.system_dir / "python"
         self.runinenv = self.system_dir / "runinenv"
@@ -351,84 +372,58 @@ class Machine(MachineBase):
         )
         self.run(f"chmod +x {self.runinenv}")
 
-    # def setup_node(self):
-    #     """Install Node from NodeSource."""
-    #     ...
+    def setup_admin(self):
+        """Install administrative interface."""
+        self.run("/root/python/bin/python3 -m venv admin")
+        self.run("runinenv admin pip install webint")
+
+    def setup_tor(self):
+        """Install Tor for anonymous routing."""
+        self._apt("install -yq tor")
+        self.torrc = "/etc/tor/torrc"
+        self.tor_data = "/var/lib/tor"
+        self.run(
+            f"cat > {self.torrc}",
+            stdin=textwrap.dedent(
+                f"""\
+                HiddenServiceDir {self.tor_data}/main
+                HiddenServicePort 80 127.0.0.1:80"""
+            ),
+        )
+        self.run("service tor restart")
+        self.onion = self.get(f"{self.tor_data}/main/hostname")
 
     def setup_nginx(self):
         """
         Install Nginx (w/ TLS, HTTPv2, RTMP) for web serving.
 
-        Additionally start under Supervisor.
-
         """
-        # version = self.versions["nginx"]
-        # nginx_src = f"nginx-{version}"
-        # mod_url = (
-        #     "https://github.com/sergey-dryabzhinsky/nginx-rtmp-module/archive/dev.zip"
-        # )
-        # with self.cd(self.src_dir) as src_dir:
-        #     src_dir.run(f"wget {mod_url} -O nginx-rtmp-module.zip")
-        #     src_dir.run("unzip -qq nginx-rtmp-module.zip")
-        # self._build(
-        #     f"nginx.org/download/{nginx_src}.tar.gz",
-        #     "--with-http_ssl_module",
-        #     "--with-http_v2_module",
-        #     f"--add-module={self.src_dir}/nginx-rtmp-module-dev",
-        #     f"--prefix={self.nginx_dir}",
-        # )
-
-        self._apt("install -yq libnginx-mod-rtmp")
+        self._apt("install -yq nginx libnginx-mod-rtmp")
+        self.nginx_conf = "/etc/nginx/nginx.conf"
+        self.web_data = "/var/www"
         self.open_ports(80, 443)
-
-        # self.run(f"mkdir -p {self.nginx_dir}/conf/conf.d")
-        # self.run(
-        #     f"cat > {self.nginx_dir}/conf/nginx.conf",
-        #     stdin=str(templates.nginx(self.address)),
-        # )
-
         protected_path = newmath.nbrandom(5)
-        self.run("mkdir /var/www")
+        self.run(f"mkdir {self.web_data}")
         self.run(
-            f"cat > /var/www/{protected_path}.html",
+            f"cat > {self.web_data}/{protected_path}.html",
             stdin=str(templates.nginx_index(self.address)),
         )
         self.run(
-            "cat > /etc/nginx/nginx.conf",
-            stdin=str(templates.nginx_conf(self.address)),
+            f"cat > {self.nginx_conf}",
+            stdin=str(templates.nginx_conf(self.address, self.onion)),
         )
-        self.run(f"service nginx restart")
-
-        # with self.supervisor(f"{self.system_dir}/servers", "servers") as config:
-        #     config["program:nginx"] = {
-        #         "autostart": "true",
-        #         "command": f"{self.nginx_dir}/sbin/nginx",
-        #         "stopsignal": "INT",
-        #         "user": "root",
-        #     }
+        self.run("service nginx restart")
         self.generate_dhparam(512)  # NOTE 2048 adds ? minutes
         return protected_path
 
+    # def setup_node(self):
+    #     """Install Node from NodeSource."""
+    #     ...
+
     def generate_dhparam(self, bits=4096):
         """Generate a unique Diffie-Hellman prime for Nginx."""
         self.run(f"openssl dhparam -out {self.system_dir}/dhparam.pem {bits}")
 
-    # def setup_tor(self):
-    #     """Install Tor for anonymous routing."""
-    #     version = self.versions["tor"]
-    #     tor_dir = f"tor-{version}"
-    #     self._build(
-    #         f"dist.torproject.org/{tor_dir}.tar.gz", f"--prefix={self.system_dir}"
-    #     )
-    #     self.run(f"mkdir -p {self.var_dir}/tor")
-    #     with self.supervisor(f"{self.system_dir}/servers", "servers") as config:
-    #         config["program:tor"] = {
-    #             "autostart": "true",
-    #             "command": f"{self.bin_dir}/tor",
-    #             "stopsignal": "INT",
-    #             "user": "gaea",
-    #         }
-
     def install_project(self, registrar, project_root=Path(".")):
         """Install sites from project in `project_root`."""
         # TODO install from Git URL
